using NUnit.Framework;
using System;

namespace Lydos.Matlab.Tests
{
    [TestFixture]
    public class ConnectionTests
    {
        #region Unit tests
        [Test]
        public void OpenSetVisibilityClose()
        {
            Engine e = new Engine();
            Assert.IsTrue(e.Connected, "Unable to connect to Matlab.");
            Assert.AreEqual(0, e.BufferSize, "Buffer size isn't initialized to zero.");
            Assert.AreEqual(string.Empty, e.Result, "Result isn't initially empty.");

            Assert.IsTrue(e.Visible, "Window isn't visible when it should be.");
            e.Visible = false;
            Assert.IsFalse(e.Visible, "Window is visible when it shouldn't be.");

            e.Close();
            Assert.IsFalse(e.Connected, "Unable to close connection to Matlab.");
        }

        [Test]
        public void MultipleConnections()
        {
            string zeroAns = "\nans =\n\n     0\n\n";
            string oneAns = "\nans =\n\n     1\n\n";
            Engine e = new Engine(128);
            Engine f = new Engine(128);

            e.Evaluate("0");
            f.Evaluate("1");

            Assert.AreEqual(zeroAns, e.Result, "First engine result buffer not expected value.");
            Assert.AreEqual(oneAns, f.Result, "Second engine result buffer not expected value.");

            e.Evaluate("m = [0 1; 2 3];");
            Assert.AreEqual(e.EvaluateToString("m"), f.EvaluateToString("m"), "Two engine connections not consitent.");

            e.Close();
            f.Close();
        }

        [Test]
        [ExpectedException(typeof(MatlabNotConnectedException))]
        public void NotConnected()
        {
            Engine e = new Engine();
            e.Close();
            e.Evaluate("0");
        }
        #endregion
    }
}
