using NUnit.Framework;
using System;

namespace Lydos.Matlab.Tests
{
    [TestFixture]
    public class ExceptionsTests
    {
        #region Fields
        Engine e = null;
        #endregion

        #region Set up and tear down
        [TestFixtureSetUp]
        public void Init()
        {
            e = new Engine();
        }

        [TestFixtureTearDown]
        public void Dispose()
        {
            e.Close();
        }
        #endregion

        #region Unit tests
        [Test]
        [ExpectedException(typeof(MatlabGetVariableFailedException))]
        public void GetUndefinedMatrix()
        {
            Matrix m = e.GetMatrix("notamatrix");
        }

        [Test]
        [ExpectedException(typeof(MatlabGetVariableFailedException))]
        public void GetUnnamedMatrix()
        {
            Matrix m = e.GetMatrix("");
        }

        [Test]
        [ExpectedException(typeof(ObjectDisposedException))]
        public void SetDisposedMatrix()
        {
            Matrix m = new Matrix(2, 2);
            m.Dispose();

            e.SetMatrix("disposedmatrix", m);
        }

        [Test]
        [ExpectedException(typeof(MatlabPutVariableFailedException))]
        public void SetUnnamedMatrix()
        {
            Matrix m = new Matrix(2, 2);

            e.SetMatrix("", m);
        }
        #endregion
    }
}
