using NUnit.Framework;
using System;

namespace Lydos.Matlab.Tests
{
    [TestFixture]
    public class InteractionTests
    {
        #region Fields
        Engine e = null;
        #endregion

        #region Set up and tear down
        [TestFixtureSetUp]
        public void Init()
        {
            e = new Engine(1024);
        }

        [TestFixtureTearDown]
        public void Dispose()
        {
            e.Close();
        }
        #endregion

        #region Unit tests
        [Test]
        public void Command()
        {
            string zeroAns = "\nans =\n\n     0\n\n";
            string oneAns = "\nans =\n\n     1\n\n";

            Assert.AreEqual(1024, e.BufferSize);
	
            Assert.AreEqual(zeroAns, e.EvaluateToString("0"));
            Assert.AreEqual(zeroAns, e.Result);

            Assert.AreEqual(zeroAns, e.EvaluateToString("1-1"));
            Assert.AreEqual(zeroAns, e.Result);

            Assert.AreEqual(zeroAns, e.EvaluateToString("sin(0)"));
            Assert.AreEqual(zeroAns, e.Result);

            Assert.AreEqual(zeroAns, e.EvaluateToString("min([0 1 2 3 4 5 6 7 8 9 10])"));
            Assert.AreEqual(zeroAns, e.Result);

            Assert.AreEqual(oneAns, e.EvaluateToString("max([0 1])"));
            Assert.AreEqual(oneAns, e.Result);
        }

        [Test]
        public void GetMatrix()
        {
            Matrix m;

            e.Evaluate("m = [0 1; 2 3];");
            m = e.GetMatrix("m");

            Assert.AreEqual(0, m[0, 0]);
            Assert.AreEqual(1, m[0, 1]);
            Assert.AreEqual(2, m[1, 0]);
            Assert.AreEqual(3, m[1, 1]);
        }

        [Test]
        public void SetMatrix()
        {
            Matrix m = new Matrix(2, 2);
            Matrix n;

            m[0, 0] = 0;
            m[0, 1] = 1;
            m[1, 0] = 2;
            m[1, 1] = 3;

            e.SetMatrix("m", m);
            n = e.GetMatrix("m");

            Assert.AreEqual(0, n[0, 0]);
            Assert.AreEqual(1, n[0, 1]);
            Assert.AreEqual(2, n[1, 0]);
            Assert.AreEqual(3, n[1, 1]);
        }

        [Test]
        public void SetMatrixFromMultiArray()
        {
            double[,] d = new double[2, 2] {{0, 1}, {2, 3}};
            Matrix n;

            e.SetMatrix("a", Matrix.FromMultiArray(d));
            GC.Collect();
            n = e.GetMatrix("a");

            Assert.AreEqual(d[0, 0], n[0, 0]);
            Assert.AreEqual(d[0, 1], n[0, 1]);
            Assert.AreEqual(d[1, 0], n[1, 0]);
            Assert.AreEqual(d[1, 1], n[1, 1]);
        }
        #endregion
    }
}
