using System;
using System.Runtime.Serialization;

namespace Lydos.Matlab
{
    /// <summary>
    /// The base class for exceptions caused by Matlab errors.
    /// </summary>
    [Serializable]
    public class MatlabException : Exception
    {
        /// <summary>
        /// Initializes a new instance of this class.
        /// </summary>
        public MatlabException() : base() { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        public MatlabException(string message) : base(message) { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message and a reference to the inner exception that is the cause of this exception.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        /// <param name="inner">The exception that is the cause of the current exception.</param>
        public MatlabException(string message, Exception inner) : base(message, inner) { }

        /// <summary>
        /// Initializes a new instance of this class with serialized data.
        /// </summary>
        /// <param name="info">The object that holds the serialized object data.</param>
        /// <param name="context">The contextual information about the source or destination.</param>
        protected MatlabException(SerializationInfo info, StreamingContext context) : base(info, context) {}
    }

    /// <summary>
    /// The exception that is thrown when attempting to use a Matrix with a non-double Matlab type.
    /// </summary>
    [Serializable]
    public class MatlabBadTypeException : MatlabException
    {
        /// <summary>
        /// Initializes a new instance of this class.
        /// </summary>
        public MatlabBadTypeException() : base() { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        public MatlabBadTypeException(string message) : base(message) { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message and a reference to the inner exception that is the cause of this exception.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        /// <param name="inner">The exception that is the cause of the current exception.</param>
        public MatlabBadTypeException(string message, Exception inner) : base(message, inner) { }

        /// <summary>
        /// Initializes a new instance of this class with serialized data.
        /// </summary>
        /// <param name="info">The object that holds the serialized object data.</param>
        /// <param name="context">The contextual information about the source or destination.</param>
        protected MatlabBadTypeException(SerializationInfo info, StreamingContext context) : base(info, context) {}
    }

    /// <summary>
    /// The exception that is thrown when a function failed to get a variable from Matlab.
    /// </summary>
    [Serializable]
    public class MatlabGetVariableFailedException : MatlabException
    {
        /// <summary>
        /// Initializes a new instance of this class.
        /// </summary>
        public MatlabGetVariableFailedException() : base() { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        public MatlabGetVariableFailedException(string message) : base(message) { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message and a reference to the inner exception that is the cause of this exception.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        /// <param name="inner">The exception that is the cause of the current exception.</param>
        public MatlabGetVariableFailedException(string message, Exception inner) : base(message, inner) { }

        /// <summary>
        /// Initializes a new instance of this class with serialized data.
        /// </summary>
        /// <param name="info">The object that holds the serialized object data.</param>
        /// <param name="context">The contextual information about the source or destination.</param>
        protected MatlabGetVariableFailedException(SerializationInfo info, StreamingContext context) : base(info, context) {}
    }

    /// <summary>
    /// The exception that is thrown when attempting to use Matlab without an open connection.
    /// </summary>
    [Serializable]
    public class MatlabNotConnectedException : MatlabException
    {
        /// <summary>
        /// Initializes a new instance of this class.
        /// </summary>
        public MatlabNotConnectedException() : base() { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        public MatlabNotConnectedException(string message) : base(message) { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message and a reference to the inner exception that is the cause of this exception.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        /// <param name="inner">The exception that is the cause of the current exception.</param>
        public MatlabNotConnectedException(string message, Exception inner) : base(message, inner) { }

        /// <summary>
        /// Initializes a new instance of this class with serialized data.
        /// </summary>
        /// <param name="info">The object that holds the serialized object data.</param>
        /// <param name="context">The contextual information about the source or destination.</param>
        protected MatlabNotConnectedException(SerializationInfo info, StreamingContext context) : base(info, context) {}
    }

    /// <summary>
    /// The exception that is thrown when Matlab failed because it ran out of unmanaged memory.
    /// </summary>
    [Serializable]
    public class MatlabOutOfMemoryException : MatlabException
    {
        /// <summary>
        /// Initializes a new instance of this class.
        /// </summary>
        public MatlabOutOfMemoryException() : base() { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        public MatlabOutOfMemoryException(string message) : base(message) { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message and a reference to the inner exception that is the cause of this exception.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        /// <param name="inner">The exception that is the cause of the current exception.</param>
        public MatlabOutOfMemoryException(string message, Exception inner) : base(message, inner) { }

        /// <summary>
        /// Initializes a new instance of this class with serialized data.
        /// </summary>
        /// <param name="info">The object that holds the serialized object data.</param>
        /// <param name="context">The contextual information about the source or destination.</param>
        protected MatlabOutOfMemoryException(SerializationInfo info, StreamingContext context) : base(info, context) {}
    }

    /// <summary>
    /// The exception that is thrown when a function failed to set a variable in Matlab.
    /// </summary>
    [Serializable]
    public class MatlabPutVariableFailedException : Exception
    {
        /// <summary>
        /// Initializes a new instance of this class.
        /// </summary>
        public MatlabPutVariableFailedException() : base() { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        public MatlabPutVariableFailedException(string message) : base(message) { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message and a reference to the inner exception that is the cause of this exception.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        /// <param name="inner">The exception that is the cause of the current exception.</param>
        public MatlabPutVariableFailedException(string message, Exception inner) : base(message, inner) { }

        /// <summary>
        /// Initializes a new instance of this class with serialized data.
        /// </summary>
        /// <param name="info">The object that holds the serialized object data.</param>
        /// <param name="context">The contextual information about the source or destination.</param>
        protected MatlabPutVariableFailedException(SerializationInfo info, StreamingContext context) : base(info, context) {}
    }

    /// <summary>
    /// The exception that is thrown when Matlab failed, but the reason is not clear.
    /// </summary>
    [Serializable]
    public class MatlabUnspecifiedException : MatlabException
    {
        /// <summary>
        /// Initializes a new instance of this class.
        /// </summary>
        public MatlabUnspecifiedException() : base() { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        public MatlabUnspecifiedException(string message) : base(message) { }

        /// <summary>
        /// Initializes a new instance of this class with a specified error message and a reference to the inner exception that is the cause of this exception.
        /// </summary>
        /// <param name="message">A string that describes the error.</param>
        /// <param name="inner">The exception that is the cause of the current exception.</param>
        public MatlabUnspecifiedException(string message, Exception inner) : base(message, inner) { }
 
        /// <summary>
        /// Initializes a new instance of this class with serialized data.
        /// </summary>
        /// <param name="info">The object that holds the serialized object data.</param>
        /// <param name="context">The contextual information about the source or destination.</param>
        protected MatlabUnspecifiedException(SerializationInfo info, StreamingContext context) : base(info, context) {}
   }
}
